Documentation
¶
Overview ¶
Package debug provide library for profiling Go program. Its a wrapper for standard runtime and runtime/pprof packages.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type CPUProfile ¶
type CPUProfile struct {
// contains filtered or unexported fields
}
CPUProfile provide a wrapper to starting and stopping CPU profiler from package "runtime/pprof".
func NewCPUProfile ¶
func NewCPUProfile(prefix string) (prof *CPUProfile)
NewCPUProfile create and start the CPU profiler. On success, it will return the running profiler; otherwise it will return nil. Do not forget to call Stop() when finished.
type MemHeap ¶
type MemHeap struct {
// RelHeapAlloc difference between heap allocation, relative to the
// first time this object initialized.
RelHeapAlloc int64
// RelHeapObjects difference between heap objects, relative to the
// first time this object initialized.
RelHeapObjects int64
// DiffHeapObject different between last heap objects and with current
// relative heap objects.
// This value is equal to MemStats.Mallocs - MemStats.Frees.
// If its positive its means the number of objects allocated,
// otherwise its represent the number of objects freed.
DiffHeapObjects int64
// contains filtered or unexported fields
}
MemHeap store the difference between heap allocation.
Source Files
¶
- cpu_profile.go
- debug.go
- memheap.go
- pprof.go
Click to show internal directories.
Click to hide internal directories.